﻿/*
 
Celerity: Sensory Overload
Copright 2012 by Adam Hill, David Gannon, Thomas Tamblyn & Patrick Ytting
Contact: adam.hill@outlook.com
Website: adamhill.org
Licence: MS-Pl Licence
  
 */

using Microsoft.Xna.Framework;
using System;

namespace Celerity.World
{
    public class Camera
    {
        public Matrix view;
        public Vector3 position;
        public Vector3 up;
        public Vector3 rotatedUp;
        public Vector3 cameraLookAt;
        public float distanceFromCentre;
        public float lookDownAngleDegrees;
        private float lookAheadDistance;

        public Vector3 headMovementScaleFactor;

        public Camera(Matrix ViewMatrix)
        {
            view = ViewMatrix;
            up = new Vector3(0.0f, 1.0f, 0.0f);
            position = Vector3.Zero;
            cameraLookAt = new Vector3(0.0f, 0.0f, 1.0f);
            distanceFromCentre = 3.0f;
            lookDownAngleDegrees = 15f;
            headMovementScaleFactor = new Vector3(3.0f, 3.0f, 3.0f);
            lookAheadDistance = 10000.0f;
        }

        public void Update(GameTime gameTime, float currentPhase, Vector3 headPos, float shipAngle)
        {
            Vector2 offset = Tunnel.GetTunnelOffset(currentPhase);
            this.position = Vector3.Transform(this.up * distanceFromCentre, Matrix.CreateRotationZ(shipAngle)) + new Vector3(-offset.X, offset.Y, 0.0f);
            Matrix cameraRotation = Matrix.CreateRotationZ(shipAngle + (float)Math.PI);
            this.rotatedUp = Vector3.Transform(this.up, cameraRotation);
            Vector3 direction = Tunnel.GetTunnelDirection(currentPhase);
            direction.X = -direction.X;

            Matrix headRotationMatrix = Matrix.CreateFromAxisAngle(this.up, (float)Math.Atan(headPos.X));
            Vector3 newHeadPos = new Vector3(-1.0f * headPos.X, -1.0f * headPos.Y, 1.0f * headPos.Z);
            newHeadPos = Vector3.Transform(newHeadPos, headRotationMatrix * cameraRotation);
            newHeadPos.X *= headMovementScaleFactor.X;
            newHeadPos.Y *= headMovementScaleFactor.Y;
            newHeadPos.Z *= headMovementScaleFactor.Z;

            //projects to a point far in the distance
            Vector3 down = new Vector3(0.0f, -(lookAheadDistance * (float)Math.Tan(lookDownAngleDegrees * (float)Math.PI / 180.0f)), 0.0f);
            this.cameraLookAt = position + (direction * lookAheadDistance) + Vector3.Transform(down, cameraRotation);

            //this.cameraLookAt = Vector3.Transform(this.cameraLookAt, cameraRotation * Matrix.CreateRotationX(lookDownAngleDegrees * (float)(Math.PI / 180)));
            view = Matrix.CreateLookAt(this.position + newHeadPos, this.cameraLookAt, this.rotatedUp);
        }
    }
}
